package de.uni_passau.fim.PADAS.group3.DataDash.Dataset;

import java.net.URL;
import java.time.LocalDate;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import de.uni_passau.fim.PADAS.group3.DataDash.category.Category;

import java.sql.Date;

import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.EnumType;
import jakarta.persistence.Enumerated;
import jakarta.persistence.GeneratedValue;
import jakarta.persistence.GenerationType;
import jakarta.persistence.Id;
import jakarta.persistence.ManyToOne;

@Entity
public class Dataset {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private UUID id;

    @Enumerated(EnumType.STRING)
    private Type type;

    private String title;
    private String summary;
    private String description;
    private String author;
    private String license;
    private Date date;
    private float rating;
    private int votes;
    private int upvotes;
    private URL url;

    @ManyToOne
    private Category category;

    @Column(name = "terms_of_use")
    private URL termsOfUse;

    private static final List<String> sortable = Arrays.asList(
        "author", "title", "upvotes", "rating", "date"
    );

    public Dataset(String title, String summary, String description, String author, URL url, Category category, Type type, String license) {
        this.rating = 0;
        this.votes = 0;
        this.upvotes = 0;
        setTitle(title);
        setSummary(summary);
        setDescription(description);
        setAuthor(author);
        setDate(LocalDate.now());
        setCategory(category);
        setType(type);
        setUrl(url);
        setLicense(license);
    }

    public Dataset() {}

    public String getSummary() {
        return summary;
    }

    public String getAuthor() {
        return author;
    }

    public Category getCategory() {
        return category;
    }

    public LocalDate getDate() {
        return date.toLocalDate();
    }

    public String getDescription() {
        return description;
    }

    public UUID getId() {
        return id;
    }

    public float getRating() {
        return rating;
    }

    public String getTitle() {
        return title;
    }

    public Type getType() {
        return type;
    }

    public int getVotes() {
        return votes;
    }

    public int getUpvotes() {
        return upvotes;
    }

    public URL getUrl() {
        return url;
    }

    public URL getTermsOfUse() {
        return termsOfUse;
    }

    public String getLicense() {
        return license;
    }

    public static List<String> getSort() {
        return sortable;
    }

    public void setSummary(String summary) {
        this.summary = summary.substring(0, Math.min(summary.length(), 100));
    }

    public void setAuthor(String author) {
        this.author = author;
    }

    public void setCategory(Category category) {
        this.category = category;
    }

    public void setDate(LocalDate localDate) {
        this.date = Date.valueOf(localDate);
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public void setUrl(URL url) {
        this.url = url;
    }

    public void setTermsOfUse(URL termsOfUse) {
        this.termsOfUse = termsOfUse;
    }

    public void setTitle(String title) {
        this.title = title.substring(0, Math.min(title.length(), 50));
    }

    public void setType(Type type) {
        this.type = type;
    }

    public void vote(int stars) {
        rating = (rating * votes + stars) / (++votes);
    }

    public void upvote() {
        upvotes++;
    }

    public void downvote() {
        upvotes--;
    }

    public void setLicense(String license) {
        this.license = license;
    }
}
