package de.uni_passau.fim.PADAS.group3.DataDash.category;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import java.util.Arrays;
import java.util.UUID;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.BDDMockito.given;
import static org.mockito.Mockito.verify;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

@WebMvcTest(CategoryController.class)
public class CategoryControllerTest {
    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private CategoryService categoryService;

    @Autowired
    private ObjectMapper objectMapper;

    Category category = new Category("yeet");

    @Test
    public void testGetAllCategories() throws Exception {
        CategoryDto category1 = CategoryDtoMapper.toDto(category);
        CategoryDto category2 = CategoryDtoMapper.toDto(category);
        given(categoryService.getAllCategories()).willReturn(Arrays.asList(category1, category2));

        mockMvc.perform(get("/api/v1/categories"))
            .andExpect(status().isOk())
            .andExpect(content().json(objectMapper.writeValueAsString(Arrays.asList(category1, category2))));
    }

    @Test
    public void testFetchCategoryByIdFound() throws Exception {
        UUID id = UUID.randomUUID();
        CategoryDto category = CategoryDtoMapper.toDto(this.category);
        given(categoryService.getCategoryById(id)).willReturn(category);

        mockMvc.perform(get("/api/v1/categories/id/{id}", id))
            .andExpect(status().isOk())
            .andExpect(content().json(objectMapper.writeValueAsString(category)));
    }

    @Test
    public void testFetchCategoryByIdNotFound() throws Exception {
        UUID id = UUID.randomUUID();
        given(categoryService.getCategoryById(id)).willReturn(null);

        mockMvc.perform(get("/api/v1/categories/id/{id}", id)).andExpect(status().isNotFound());
    }

    @Test
    public void testCreateCategory() throws Exception {
        CategoryDto categoryDto = CategoryDtoMapper.toDto(category);
        mockMvc.perform(post("/api/v1/categories")
            .contentType(MediaType.APPLICATION_JSON)
            .content(objectMapper.writeValueAsString(categoryDto)))
            .andExpect(status().isCreated());

        // Verify that the service method was called once
        verify(categoryService).addCategory(any(CategoryDto.class));
    }
}
